import { ApplicationHandler } from '../application/ApplicationHandler';
import { State } from '../const';
import { ApplicationRepresentation } from '../domain';
import { DaemonIntrospection } from './DaemonIntrospection';
/**
 * Class Daemon
 */
export declare class Daemon {
    private messengerServer;
    private monitor;
    private introspection;
    private daemonLogger;
    private globalConfigProcesser;
    private globalConfig;
    state: State;
    apps: Map<any, ApplicationHandler>;
    constructor();
    /**
     * Start Daemon
     * @return {Promise<any>}
     */
    start(): Promise<{}>;
    /**
     * Handing all passive exit events, such as process signal, uncaughtException
     */
    handleExit(): void;
    /**
     * Start an application
     *
     * @param {ApplicationRepresentation} applicationRepresentation
     * @returns {Promise<ApplicationHandler>}
     */
    startApp(applicationRepresentation: ApplicationRepresentation): Promise<ApplicationHandler>;
    /**
     * Reload an application
     * @param appName
     * @param processName
     * @return {Promise<ApplicationHandler>}
     */
    reloadApp(appName: any, processName?: any): Promise<ApplicationHandler>;
    /**
     * stop an application
     * @param appName
     * @return {Promise<ApplicationHandler>}
     */
    stopApp(appName: any): Promise<ApplicationHandler>;
    /**
     * stop all the applications
     * @return {Promise<void>}
     */
    stopAllApps(): Promise<void>;
    /**
     * stop an application
     * @return {Promise<void>}
     */
    stop(): Promise<void>;
    /**
     * Start the monitor
     * @return {Promise<void>}
     */
    private startMonitor();
    /**
     * Stop the monitor
     * @return {Promise<void>}
     */
    private stopMonitor();
    /**
     * Handle daemon's command invocations
     * @param message
     * @param reply
     */
    handleCommand(message: any, reply: any): Promise<void>;
    getIntrospection(): DaemonIntrospection;
}
