'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const const_1 = require("../const");
const GlobalConfigProcessor_1 = require("./GlobalConfigProcessor");
const extend = require('extend');
function calcAppName(dir) {
    let ret;
    try {
        const pkg = require(path_1.join(process.cwd(), 'package.json'));
        pkg.name.toString();
        ret = removePkgNameScope(pkg.name);
    }
    catch (err) {
        ret = null;
    }
    if (!ret && dir) {
        ret = dir.split('/').slice(-1)[0] || null;
    }
    return ret;
}
exports.calcAppName = calcAppName;
function removePkgNameScope(pkgName) {
    return pkgName.replace(/^@[^\/]*\//, '');
}
exports.removePkgNameScope = removePkgNameScope;
function attachEntryParams(command, cliConfig, defaultConfig = {}) {
    const currentPath = process.cwd();
    let pandoraConfig;
    try {
        const pandoraAttachConfig = require(`${currentPath}/package.json`)['pandora'];
        pandoraConfig = pandoraAttachConfig[command] || {};
        pandoraConfig['config'] = pandoraConfig['config'] || [];
        // set global config to environment
        pandoraConfig['config'].unshift(process.env[const_1.PANDORA_GLOBAL_CONFIG] || '');
        process.env[const_1.PANDORA_GLOBAL_CONFIG] = pandoraConfig['config'].filter((text) => {
            return !!text;
        }).join(GlobalConfigProcessor_1.GlobalConfigProcessor.GLOBAL_PACKAGE_SPLIT);
    }
    catch (err) {
        // console.log(err);
        pandoraConfig = {};
    }
    const sendConfig = extend(true, defaultConfig, pandoraConfig || {}, cliConfig);
    sendConfig['entry'] = sendConfig['entry'] || currentPath;
    const p = path_1.resolve(sendConfig['entry']);
    const fd = fs_1.statSync(p);
    if (fd.isDirectory()) {
        sendConfig['appDir'] = p;
    }
    else if (fd.isFile()) {
        const dir = path_1.dirname(sendConfig['entry']);
        throw new Error(`Pandora.js can only start a Pandora.js project directory, like [ pandora ${command}${dir === '.' ? ' ./' : ' ' + dir + '/'} ]\nYou can use [ pandora init ${sendConfig['entry']} ] to init a Pandora.js project`);
        // sendConfig['entryFile'] = p;
        // sendConfig['appDir'] = currentPath;
    }
    if (cliConfig.name) {
        sendConfig.appName = cliConfig.name;
    }
    if (sendConfig.env) {
        try {
            const envMap = {};
            const splitEnv = sendConfig.env.split(' ');
            for (const item of splitEnv) {
                const execRes = /^(.*?)=(.*)$/.exec(item);
                const key = execRes[1];
                const value = execRes[2];
                envMap[key] = value;
            }
            sendConfig.globalEnv = envMap;
        }
        catch (err) {
            console.error(err);
        }
    }
    if (sendConfig['args']) {
        sendConfig.globalArgs = sendConfig.args.split(' ');
    }
    if (sendConfig['node-args']) {
        sendConfig.globalExecArgv = sendConfig['node-args'].split(' ');
    }
    for (const optName of ['inspect-port', 'inspect']) {
        if (sendConfig.hasOwnProperty(optName)) {
            const inspect = sendConfig[optName];
            if (optName === 'inspect' && inspect === '' || true === inspect) {
                sendConfig['inspector'] = true;
            }
            else if (typeof inspect === 'string' && inspect) {
                const { host, port } = parseInspectPort(inspect);
                sendConfig['inspector'] = { host, port, setPortOnly: optName === 'inspect-port' };
            }
            break;
        }
    }
    const sendConfig2nd = {};
    for (const key of ['appName', 'appDir', 'inspector',
        'globalEnv', 'globalArgs', 'globalExecArgv']) {
        if (sendConfig.hasOwnProperty(key)) {
            sendConfig2nd[key] = sendConfig[key];
        }
    }
    return sendConfig2nd;
}
exports.attachEntryParams = attachEntryParams;
function parseInspectPort(inspect) {
    const split = inspect.split(':');
    let port, host;
    if (split.length >= 2) {
        host = split[0];
        port = parseInt(split[1], 10);
    }
    else if (inspect.indexOf('.') > -1) {
        host = inspect;
    }
    else {
        port = parseInt(inspect, 10);
    }
    return { host, port };
}
exports.parseInspectPort = parseInspectPort;
//# sourceMappingURL=Helpers.js.map